################################################################################
###  This sample uses the tam DSL to create a tile assembly system which     ###
###  self-assembles into a log-width binary counter (with a seed row         ###
###  representing the value 1 but two tiles wide to provide a simpler        ###
###  example - all subsequent values are represented in rows of the correct  ###
###  width.  This sample is the same as in counter_using_signal_objects.py,  ###
###  but it instead defines all MultisignalType objects implicitly.          ###
################################################################################

from __future__ import print_function, division
import tam

N = tam.Direction.North
S = tam.Direction.South
E = tam.Direction.East
W = tam.Direction.West

if __name__ == '__main__':

    lsbseed = tam.Tile(name='lsbseed', label='1', tilecolor='red', textcolor='black', westglue=('seed', 2))
    msbseed = tam.Tile(name='msbseed', label='0', tilecolor='red', textcolor='black', eastglue=('seed', 2))
    
    lsb = tam.TileTemplate(name='lsb')
    interior = tam.TileTemplate(name='int')
    msb = tam.TileTemplate(name='msb')
    grow = tam.TileTemplate(name='grow')
    new = tam.TileTemplate(name='new')
    
    tst = tam.TileSetTemplate()

    tst.join(2, N, lsbseed, lsb, bit=1)
    tst.join(1, N, msbseed, msb, bit=0)
    tst.join(2, N, lsb, lsb, bit=(0,1))
    tst.join(1, W, lsb, interior, carry=(0,1))
    tst.join(1, N, interior, interior, bit=(0,1))
    tst.join(1, W, interior, interior, carry=(0,1))
    tst.join(1, N, grow, interior, bit=0)
    tst.join(1, W, interior, msb, carry=(0,1))
    tst.join(1, W, interior, grow, carry=1)
    tst.join(1, N, msb, msb, bit=(0,1))
    tst.join(1, N, msb, grow, bit=1)
    tst.join(2, W, grow, new, carry=1)
    tst.join(1, N, new, msb, bit=1)
    
    def nextBitAndCarry(bit, carry):
        return ((bit + carry) % 2 , (bit + carry) // 2)

    interior.addTransition(inputs=('bit', 'carry'), outputs=('bit', 'carry'), function=nextBitAndCarry)
    lsb.addTransition(inputs=('bit'), outputs=('bit', 'carry'), expression='(1-bit,bit)')
    msb.addTransition(inputs=('bit','carry'), outputs=('bit'), expression='(bit+carry)%2')
    
    tst.setChooser(grow, inputs=('bit','carry'), expression='"grow" if bit == 1 and carry == 1 else "msb"')

    lsb.setLabelFunction(outputs=['bit'], inputs=[], expression='str(bit)')
    interior.setLabelFunction(outputs=['bit'], inputs=[], expression='str(bit)')
    msb.setLabelFunction(outputs=['bit'], inputs=[], expression='str(bit)')
    grow.setLabelFunction(outputs=['bit'], inputs=[], expression='str(bit)')
    new.setLabelFunction(outputs=['bit'], inputs=[], expression='str(bit)')

    tiles = tst.createTiles()

    tileSystem = tam.TileSystem('log-width-counter', tiles)
    tileSystem.addToSeedAssembly((1,0), lsbseed)
    tileSystem.addToSeedAssembly((0,0), msbseed)

    tileSystem.writeToFiles('log-width-counter')
